@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,11 +15,16 @@ from gi.repository import WebKit2, Gtk, Gio, GLib
 from urllib.parse import urlparse
 from time import time
 
-from eolie.define import App, Indicator, LoadingType
+from eolie.define import App, LoadingType
+from eolie.context import Context
 from eolie.webview_errors import WebViewErrors
 from eolie.webview_navigation import WebViewNavigation
 from eolie.webview_signals import WebViewSignals
 from eolie.webview_artwork import WebViewArtwork
+from eolie.webview_state import WebViewState
+from eolie.webview_credentials import WebViewCredentials
+from eolie.webview_helpers import WebViewHelpers
+from eolie.webview_night_mode import WebViewNightMode
 from eolie.list import LinkedList
 from eolie.logger import Logger
 
@@ -29,34 +34,32 @@ class WebView(WebKit2.WebView):
         WebKit view
     """
 
-    def new(window, view):
+    def new(window):
         """
             New webview
             @param window as Window
-            @param view as View
         """
-        # context = WebKit2.WebContext.new()
-        # Context(context)
-        context = WebKit2.WebContext.get_default()
-        webview = WebKit2.WebView.new_with_context(context)
-        content_manager = webview.get_property("user-content-manager")
+        content_manager = WebKit2.UserContentManager.new()
+        for content_filter in App().content_filters:
+            if content_filter is not None:
+                content_manager.add_filter(content_filter)
+
+        webview = WebKit2.WebView.new_with_user_content_manager(
+            content_manager)
         webview.__class__ = WebViewMeta
-        webview.__init(None, context, content_manager, window, view)
+        webview.__init(None, window)
         return webview
 
-    def new_ephemeral(window, view):
+    def new_ephemeral(window):
         """
             New ephemeral webview
             @param window as Window
-            @param view as View
         """
-        # context = WebKit2.WebContext.new_ephemeral()
-        # Context(context)
-        context = App().ephemeral_context
+        context = WebKit2.WebContext.new_ephemeral()
+        Context(context)
         webview = WebKit2.WebView.new_with_context(context)
         webview.__class__ = WebViewMeta
-        content_manager = webview.get_property("user-content-manager")
-        webview.__init(None, context, content_manager, window, view)
+        webview.__init(None, window)
         return webview
 
     def new_with_related_view(related, window):
@@ -68,10 +71,7 @@ class WebView(WebKit2.WebView):
         """
         webview = WebKit2.WebView.new_with_related_view(related)
         webview.__class__ = WebViewMeta
-        # Related view are linked to related context, can't do anything about
-        # this
-        webview.__init(related, related.context,
-                       related.content_manager, window, None)
+        webview.__init(related, window)
         return webview
 
     def set_setting(self, key, value):
@@ -86,18 +86,25 @@ class WebView(WebKit2.WebView):
         else:
             settings.set_property(key, value)
 
+    def reload_bypass_cache(self):
+        """
+            Reload page, destroy night mode cache
+        """
+        self.remove_night_mode_cache()
+        WebKit2.WebView.reload_bypass_cache(self)
+
     def update_zoom_level(self):
         """
             Update zoom level
         """
         try:
-            zoom_level = self._window.zoom_level
-            if self._related_view is not None:
-                window = self._related_view.get_ancestor(Gtk.Window)
+            zoom_level = self.window.zoom_level
+            if self.__related is not None:
+                window = self.__related.get_ancestor(Gtk.Window)
                 if window is not None and hasattr(window, "zoom_level"):
                     zoom_level = window.zoom_level
             else:
-                _zoom_level = App().websettings.get_zoom(self.uri)
+                _zoom_level = App().websettings.get("zoom", self.uri)
                 if _zoom_level is not None:
                     zoom_level = _zoom_level / 100
         except Exception as e:
@@ -117,11 +124,11 @@ class WebView(WebKit2.WebView):
             Zoom in view
             @return current zoom after zoom in
         """
-        current = App().websettings.get_zoom(self.uri)
+        current = App().websettings.get("zoom", self.uri)
         if current is None:
-            current = int(self._window.zoom_level * 100)
+            current = int(self.window.zoom_level * 100)
         current += 10
-        App().websettings.set_zoom(current, self.uri)
+        App().websettings.set("zoom", self.uri, current)
         self.update_zoom_level()
         return current
 
@@ -130,13 +137,13 @@ class WebView(WebKit2.WebView):
             Zoom out view
             @return current zoom after zoom out
         """
-        current = App().websettings.get_zoom(self.uri)
+        current = App().websettings.get("zoom", self.uri)
         if current is None:
-            current = int(self._window.zoom_level * 100)
+            current = int(self.window.zoom_level * 100)
         current -= 10
         if current == 0:
             return 10
-        App().websettings.set_zoom(current, self.uri)
+        App().websettings.set("zoom", self.uri, current)
         self.update_zoom_level()
         return current
 
@@ -145,28 +152,11 @@ class WebView(WebKit2.WebView):
             Reset zoom level
             @return current zoom after zoom out
         """
-        current = int(self._window.zoom_level * 100)
-        App().websettings.set_zoom(None, self.uri)
+        current = int(self.window.zoom_level * 100)
+        App().websettings.set("zoom", self.uri, None)
         self.update_zoom_level()
         return current
 
-    def set_title(self, title):
-        """
-            Set title, will be returned by title property until title is set
-            by WebView
-            @param title as str
-        """
-        self._title = title
-        self.emit("title-changed", title)
-
-    def set_uri(self, uri):
-        """
-            Set delayed uri
-            @param uri as str
-        """
-        self._uri = uri
-        self.emit("uri-changed", uri)
-
     def update_spell_checking(self, uri):
         """
             Update spell checking
@@ -247,32 +237,16 @@ class WebView(WebKit2.WebView):
             @param uri as uri
             @param loading_type as Gdk.LoadingType
         """
-        parsed = urlparse(uri)
-        if App().settings.get_value("adblock") and\
-                parsed.scheme in ["http", "https"] and\
-                not App().adblock_exceptions.find_parsed(parsed):
-            if App().adblock.is_netloc_blocked(parsed.netloc) or\
-                    App().adblock.is_uri_blocked(uri,
-                                                 parsed.netloc):
-                Logger.debug("WebView::new_page(): blocking %s ->%s",
-                             uri, self.uri)
-                return True
-        if loading_type == LoadingType.POPOVER:
-            if self.ephemeral:
-                webview = WebView.new_ephemeral(self._window, None)
-            else:
-                webview = WebView.new(self._window, None)
-            self._window.container.popup_webview(webview, True)
-            GLib.idle_add(webview.load_uri, uri)
+        if self.is_ephemeral:
+            webview = WebView.new_ephemeral(self.window)
         else:
-            self.__new_pages_opened += 1
-            webview = self._window.container.add_webview(
-                uri,
-                loading_type,
-                self.ephemeral,
-                None,
-                self.atime -
-                self.__new_pages_opened)
+            webview = WebView.new(self.window)
+        webview.show()
+        webview.load_uri(uri)
+        if loading_type == LoadingType.POPOVER:
+            self.window.container.popup_webview(webview)
+        else:
+            self.window.container.add_webview(webview, loading_type)
             webview.set_parent(self)
             self.add_child(webview)
 
@@ -290,20 +264,6 @@ class WebView(WebKit2.WebView):
         """
         self.__atime = atime
 
-    def set_view(self, view):
-        """
-            Set webview view
-            @param view as View
-        """
-        self.__view = view
-
-    def set_window(self, window):
-        """
-            Set webview window
-            @param window as Window
-        """
-        self._window = window
-
     def add_child(self, child):
         """
             Add a child to webview
@@ -311,38 +271,29 @@ class WebView(WebKit2.WebView):
         """
         if child not in self.__children:
             self.__children.insert(0, child)
+            child.connect("destroy", self.__on_child_destroy)
 
-    def remove_child(self, child):
+    def set_shown(self, shown):
         """
-            Remove child from webview
-            @param child as WebView
+            Set webview as shown
+            @param shown as bool
         """
-        if child in self.__children:
-            self.__children.remove(child)
+        self.__shown = shown
+
+    def set_window(self, window):
+        """
+            Update GTK window
+            @param window as Window
+        """
+        self.__window = window
 
     @property
-    def content_manager(self):
-        """
-            Get content manager
-            @return WebKit2.UserContentManager
-        """
-        return self.__content_manager
-
-    @property
-    def readable(self):
+    def readability(self):
         """
             True if webview readable
             @return bool
         """
-        return self._readable
-
-    @property
-    def context(self):
-        """
-            Get context
-            @return WebKit2.WebContext
-        """
-        return self.__context
+        return self._readability
 
     @property
     def atime(self):
@@ -368,41 +319,13 @@ class WebView(WebKit2.WebView):
         """
         return self.__parent
 
-    @property
-    def view(self):
-        """
-            Get view
-            @return View
-        """
-        return self.__view
-
     @property
     def shown(self):
         """
             True if page already shown on screen (one time)
+            @return bool
         """
-        return self._shown
-
-    @property
-    def title(self):
-        """
-            Get title (loaded or unloaded)
-            @return str
-        """
-        title = self.get_title()
-        if title is None:
-            title = self._title
-        if title is None:
-            title = self.uri
-        return title
-
-    @property
-    def uri(self):
-        """
-            Get uri (loaded or unloaded)
-            @return str
-        """
-        return self._uri
+        return self.__shown
 
     @property
     def netloc(self):
@@ -415,7 +338,7 @@ class WebView(WebKit2.WebView):
         return parsed.netloc or ""
 
     @property
-    def ephemeral(self):
+    def is_ephemeral(self):
         """
             True if view is private/ephemeral
             @return bool
@@ -423,50 +346,50 @@ class WebView(WebKit2.WebView):
         return self.get_property("is-ephemeral")
 
     @property
-    def selection(self):
+    def related(self):
         """
-            Get current selection
-            @return str
+            Get related webview
+            @return WebView
         """
-        return self.__selection
+        return self.__related
+
+    @property
+    def window(self):
+        """
+            Get window
+            @return Gtk.Window
+        """
+        return self.__window
 
 #######################
 # PRIVATE             #
 #######################
-    def __init(self, related_view, context, content_manager, window, view):
+    def __init(self, related, window):
         """
             Init WebView
-            @param related_view as WebView
-            @param context as WebKit2.WebContext
-            @param content_manager as WebKit2.UserContentManager
+            @param related as WebView
             @param window as Window
-            @param view as View
         """
+        WebViewHelpers.__init__(self)
+        WebViewNightMode.__init__(self)
+        WebViewState.__init__(self)
         WebViewErrors.__init__(self)
         WebViewNavigation.__init__(self)
         WebViewSignals.__init__(self)
-        self.__context = context
         WebViewArtwork.__init__(self)
-        self._window = window
-        self.__view = view
-        self.__content_manager = content_manager
+        WebViewCredentials.__init__(self)
+        self.__window = window
         self.__atime = 0
         self.__children = []
         self.__parent = None
-        self.__new_pages_opened = 0
-        # WebKitGTK doesn't provide an API to get selection, so try to guess
-        # it from clipboard FIXME Get it from extensions
-        self.__selection = ""
-        self._readable = False
-        self._uri = None
-        self._initial_uri = None
+        self._readability = False
         self._title = None
-        self._related_view = related_view
-        self._shown = False
+        self.__related = related
+        self.__shown = False
         self.set_hexpand(True)
         self.set_vexpand(True)
         self.clear_text_entry()
-        if related_view is None:
+        if related is None:
             # Set settings
             settings = self.get_settings()
             settings.set_property("enable-java",
@@ -489,8 +412,8 @@ class WebView(WebKit2.WebView):
                                       App().settings.get_value(
                                           "font-serif").get_string())
             settings.set_property("auto-load-images", True)
-            settings.set_hardware_acceleration_policy(
-                WebKit2.HardwareAccelerationPolicy.NEVER)
+            # settings.set_hardware_acceleration_policy(
+            #    WebKit2.HardwareAccelerationPolicy.NEVER)
             settings.set_property("enable-site-specific-quirks", True)
             settings.set_property("allow-universal-access-from-file-urls",
                                   False)
@@ -503,8 +426,10 @@ class WebView(WebKit2.WebView):
             if not autoplay_videos:
                 settings.set_property("media-playback-requires-user-gesture",
                                       True)
-            settings.set_property("enable-developer-extras",
-                                  App().settings.get_value("developer-extras"))
+            if App().settings.get_value("developer-extras"):
+                settings.set_property("enable-developer-extras", True)
+                settings.set_enable_write_console_messages_to_stdout(
+                    App().settings.get_value("debug"))
             settings.set_property("enable-offline-web-application-cache", True)
             settings.set_property("enable-page-cache", True)
             settings.set_property("enable-resizable-text-areas", True)
@@ -522,6 +447,7 @@ class WebView(WebKit2.WebView):
                     "enable-back-forward-navigation-gestures", True)
         self.connect("create", self.__on_create)
         self.connect("load-changed", self._on_load_changed)
+        self.connect("destroy", self.__on_destroy)
 
     def __set_system_fonts(self, settings):
         """
@@ -546,7 +472,7 @@ class WebView(WebKit2.WebView):
             @param related as WebView
             @param navigation_action as WebKit2.NavigationAction
         """
-        webview = WebView.new_with_related_view(related, self._window)
+        webview = WebView.new_with_related_view(related, self.window)
         webview.set_atime(related.atime - 1)
         elapsed = time() - related._last_click_time
         uri = navigation_action.get_request().get_uri()
@@ -557,6 +483,7 @@ class WebView(WebKit2.WebView):
                         elapsed)
         webview.set_uri(uri)
         webview.set_parent(self)
+        webview.show()
         self.add_child(webview)
         return webview
 
@@ -568,43 +495,37 @@ class WebView(WebKit2.WebView):
             @param navigation_action as WebKit2.NavigationAction
             @param elapsed as float
         """
-        popup_block = App().settings.get_value("popupblock")
-        parsed = urlparse(webview.uri)
-        exception = elapsed < 1.0 or\
-            App().popup_exceptions.find_parsed(parsed)
         properties = webview.get_window_properties()
-        if not exception and popup_block and\
-                navigation_action.get_navigation_type() in [
-                    WebKit2.NavigationType.OTHER,
-                    WebKit2.NavigationType.RELOAD,
-                    WebKit2.NavigationType.BACK_FORWARD]:
-            related.add_popup(webview)
-            webview.connect("close", self.__on_popup_close, related)
-            if related == self._window.container.current.webview:
-                self._window.toolbar.title.show_indicator(
-                    Indicator.POPUPS)
-        elif (properties.get_toolbar_visible() or
+        if (properties.get_toolbar_visible() or
                 properties.get_scrollbars_visible()):
-            self._window.container.add_webview_with_new_view(
+            self.window.container.add_webview(
                 webview,
                 LoadingType.FOREGROUND)
         else:
-            self._window.container.popup_webview(webview, True)
+            self.window.container.popup_webview(webview)
 
-    def __on_popup_close(self, webview, related):
+    def __on_destroy(self, webview):
         """
-            Remove webview from popups
+            Prevent GC
             @param webview as WebView
-            @param related as WebView
         """
-        related.remove_popup(webview)
-        if self._window.container.current.webview == related and\
-                not related.popups:
-            self._window.toolbar.title.show_indicator(Indicator.NONE)
+        self.__parent = None
+        self.__related = None
+        for child in self.__children:
+            child.disconnect_by_func(self.__on_child_destroy)
+
+    def __on_child_destroy(self, webview):
+        """
+            Remove from children
+            @param webview as WebView
+        """
+        if webview in self.__children:
+            self.__children.remove(webview)
 
 
 class WebViewMeta(WebViewNavigation, WebView, WebViewErrors,
-                  WebViewSignals, WebViewArtwork):
+                  WebViewSignals, WebViewArtwork, WebViewState,
+                  WebViewCredentials, WebViewHelpers, WebViewNightMode):
 
     def __init__(self):
         pass
@@ -618,10 +539,8 @@ class WebViewMeta(WebViewNavigation, WebView, WebViewErrors,
             @param webview as WebView
             @param event as WebKit2.LoadEvent
         """
+        WebViewCredentials._on_load_changed(self, webview, event)
+        WebViewHelpers._on_load_changed(self, webview, event)
         WebViewNavigation._on_load_changed(self, webview, event)
-        WebViewSignals._on_load_changed(self, webview, event)
         WebViewArtwork._on_load_changed(self, webview, event)
-        if event == WebKit2.LoadEvent.STARTED:
-            self.__new_pages_opened = 0
-        elif event == WebKit2.LoadEvent.COMMITTED:
-            self._uri = webview.get_uri()
+        WebViewNightMode._on_load_changed(self, webview, event)
