@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -12,30 +12,32 @@
 
 from gi.repository import Gtk, GLib, Gio, Gdk
 
-from eolie.define import App, Indicator, LoadingType
+from eolie.define import App, LoadingType
 from eolie.toolbar import Toolbar
 from eolie.container import Container
-from eolie.utils import get_current_monitor_model, name_from_profile_id
+from eolie.utils import get_current_monitor_model, emit_signal
 from eolie.helper_task import TaskHelper
 from eolie.logger import Logger
+from eolie.window_state import WindowState
 
 
-class Window(Gtk.ApplicationWindow):
+class Window(Gtk.ApplicationWindow, WindowState):
     """
         Main window
     """
 
-    def __init__(self, app, size, maximized):
+    def __init__(self, size, is_maximized):
         """
             Init window
             @param app as Gtk.Application
             @param size as (int, int)
-            @param maximized as bool
+            @param is_maximized as bool
         """
         Gtk.ApplicationWindow.__init__(self,
-                                       application=app,
+                                       application=App(),
                                        title="Eolie",
                                        icon_name="org.gnome.Eolie")
+        WindowState.__init__(self)
         self.__monitor_model = ""
         self.__popovers = []
         self.__zoom_level = 1.0
@@ -46,11 +48,14 @@ class Window(Gtk.ApplicationWindow):
         self.__size = size
         self.set_default_size(size[0], size[1])
         self.__setup_content()
-        self.connect("realize", self.__on_realize, maximized)
-        self.connect("key-press-event", self.__on_key_press_event)
-        self.connect("key-release-event", self.__on_key_release_event)
+        self.connect("realize", self.__on_realize, is_maximized)
         self.connect("window-state-event", self.__on_window_state_event)
         self.connect("configure-event", self.__on_configure_event)
+        self.__key_event_controller = Gtk.EventControllerKey.new(self)
+        self.__key_event_controller.connect("key-pressed",
+                                            self.__on_key_pressed)
+        self.__key_event_controller.connect("key-released",
+                                            self.__on_key_released)
 
         # Set window actions
         shortcut_action = Gio.SimpleAction.new("shortcut",
@@ -64,14 +69,17 @@ class Window(Gtk.ApplicationWindow):
             Update zoom level
             @param force as bool
         """
-        monitor_model = get_current_monitor_model(self)
-        if force or monitor_model != self.__monitor_model:
-            # Update window default zoom level
-            self.__update_zoom_level()
-            self.__monitor_model = monitor_model
-            # Update view zoom level
-            for view in self.__container.views:
-                view.webview.update_zoom_level()
+        try:
+            monitor_model = get_current_monitor_model(self)
+            if force or monitor_model != self.__monitor_model:
+                # Update window default zoom level
+                self.__update_zoom_level()
+                self.__monitor_model = monitor_model
+                # Update view zoom level
+                for webview in self.__container.webviews:
+                    webview.update_zoom_level()
+        except Exception as e:
+            Logger.error("Window::update_zoom_level(): %s", e)
 
     def fullscreen(self, force=True):
         """
@@ -81,16 +89,15 @@ class Window(Gtk.ApplicationWindow):
         if self.__fullscreen_revealer is not None:
             return
         self.__fullscreen_toolbar = Toolbar(self, True)
-        # Do not count container views as destroy may be pending on somes
+        # Do not count container.webviews as destroy may be pending on somes
         count = str(len(self.__container.pages_manager.children))
         self.__fullscreen_toolbar.actions.count_label.set_text(count)
         self.__fullscreen_toolbar.show()
-        self.update(self.container.current.webview)
         self.__fullscreen_revealer = Gtk.Revealer.new()
         self.__fullscreen_revealer.set_property("valign", Gtk.Align.START)
         self.__fullscreen_revealer.add(self.__fullscreen_toolbar)
         self.__fullscreen_revealer.show()
-        self.__container.add_overlay(self.__fullscreen_revealer)
+        self.__container.overlay.add_overlay(self.__fullscreen_revealer)
         self.__container.sites_manager.hide()
         if force:
             Gtk.ApplicationWindow.fullscreen(self)
@@ -110,56 +117,17 @@ class Window(Gtk.ApplicationWindow):
             self.__container.sites_manager.show()
         self.__fullscreen_toolbar = None
         self.__fullscreen_revealer = None
-        self.update(self.container.current.webview)
-        # Do not count container views as destroy may be pending on somes
+        # Do not count container.webviews as destroy may be pending on somes
         # Reason: we do not remove/destroy view to let stack animation run
         count = len(self.container.pages_manager.children)
         self.toolbar.actions.count_label.set_text(str(count))
         if force:
             Gtk.ApplicationWindow.unfullscreen(self)
-        self.__container.current.webview.run_javascript(
+        self.__container.webview.run_javascript(
             "document.webkitExitFullscreen();",
             None,
             None)
 
-    def update(self, webview):
-        """
-            Update window
-            @param webview as WebView
-        """
-        uri = webview.uri
-        if uri is not None:
-            self.toolbar.title.set_uri(uri)
-            accept_tls = App().websettings.get_accept_tls(uri)
-            self.toolbar.end.show_tls_button(accept_tls)
-        if webview.popups:
-            self.toolbar.title.show_indicator(Indicator.POPUPS)
-        else:
-            self.toolbar.title.show_indicator(Indicator.NONE)
-        if webview.is_loading():
-            self.toolbar.title.set_loading(True)
-            self.toolbar.title.progress.show()
-        else:
-            self.toolbar.title.progress.hide()
-            self.toolbar.title.set_loading(False)
-            readable_content = webview.readable_content
-            self.toolbar.title.show_readable_button(readable_content != "")
-        profile = name_from_profile_id(webview.profile)
-        title = webview.title
-        if title:
-            self.toolbar.title.set_title(profile, title)
-        elif uri:
-            self.toolbar.title.set_title(profile, uri)
-        self.toolbar.actions.set_actions(webview)
-
-    def hide(self):
-        """
-            Hide window, save sidebar width and clean up
-        """
-        self.disconnect_by_func(self.__on_window_state_event)
-        self.disconnect_by_func(self.__on_configure_event)
-        Gtk.ApplicationWindow.hide(self)
-
     def register(self, popover, monitor=True):
         """
             Add a popover to window
@@ -234,19 +202,10 @@ class Window(Gtk.ApplicationWindow):
         """
         return self.__window_state & Gdk.WindowState.FULLSCREEN
 
-    @property
-    def modifiers(self):
-        """
-            Get modifier
-            return int
-        """
-        # https://bugs.webkit.org/show_bug.cgi?id=181041
-        return self.__modifiers
-
     @property
     def size(self):
         """
-            Unmaximized window size
+            Unis_maximized window size
             return (int, int)
         """
         return self.__size
@@ -285,13 +244,13 @@ class Window(Gtk.ApplicationWindow):
         self.__toolbar.headerbar.set_show_close_button(True)
         self.add(self.__container)
 
-    def __setup_window(self, maximized):
+    def __setup_window(self, is_maximized):
         """
             Set window
-            @param maximized as bool
+            @param is_maximized as bool
         """
         try:
-            if maximized:
+            if is_maximized:
                 # Lets resize happen
                 GLib.idle_add(self.maximize)
         except Exception as e:
@@ -371,43 +330,51 @@ class Window(Gtk.ApplicationWindow):
             if not lock:
                 self.__fullscreen_revealer.set_reveal_child(False)
 
-    def __on_realize(self, widget, maximized):
+    def __on_realize(self, widget, is_maximized):
         """
             Update zoom level
             @param widget as Gtk.Widget
-            @param maximized as bool
+            @param is_maximized as bool
         """
-        self.__setup_window(maximized)
+        self.__setup_window(is_maximized)
         self.update_zoom_level(False)
 
-    def __on_key_press_event(self, window, event):
+    def __on_key_pressed(self, controller, keyval, keycode, state):
         """
             Update PagesManager sort on Ctrl<Tab>
-            @param window as Window
+            @param controller as Gtk.EventControllerKey
+            @param keyval as int
+            @param keycode as int
+            @param state as Gtk.ModifierType
             @param event as Gdk.EventKey
         """
-        if event.keyval in [Gdk.KEY_Shift_L, Gdk.KEY_Control_L]:
-            self.__modifiers = event.keyval
-        if event.state & Gdk.ModifierType.CONTROL_MASK and\
-                event.keyval == Gdk.KEY_Tab:
-            if not self.container.in_expose:
+        if state & Gdk.ModifierType.CONTROL_MASK:
+            if keyval == Gdk.KEY_Tab and not self.container.in_expose:
                 self.container.pages_manager.update_sort()
+            elif keyval == Gdk.KEY_z:
+                self.container.webview.run_javascript_from_gresource(
+                    "/org/gnome/Eolie/javascript/HandleInputPrev.js",
+                    None, None)
+            elif keyval == Gdk.KEY_Z:
+                self.container.webview.run_javascript_from_gresource(
+                    "/org/gnome/Eolie/javascript/HandleInputNext.js",
+                    None, None)
 
-    def __on_key_release_event(self, window, event):
+    def __on_key_released(self, controller, keyval, keycode, state):
         """
             Handle Esc/Ctrl release
-            @param window as Window
+            @param controller as Gtk.EventControllerKey
+            @param keyval as int
+            @param keycode as int
+            @param state as Gtk.ModifierType
             @param event as Gdk.EventKey
         """
-        self.__modifiers = 0
-        if event.keyval == Gdk.KEY_Control_L:
+        if keyval == Gdk.KEY_Control_L:
             self.__container.ctrl_released()
-        elif event.keyval == Gdk.KEY_Escape:
+        elif keyval == Gdk.KEY_Escape:
             self.__container.set_expose(False)
-            if self.__container.current is not None and\
-                    self.__container.current.reading:
-                self.__container.current.switch_read_mode()
-                self.__toolbar.title.set_reading()
+            if self.__container.reading:
+                self.__container.toggle_reading()
 
     def __on_shortcut_action(self, action, param):
         """
@@ -419,7 +386,7 @@ class Window(Gtk.ApplicationWindow):
         if string == "uri":
             if self.is_fullscreen:
                 self.__fullscreen_revealer.set_reveal_child(True)
-            self.toolbar.title.focus_entry()
+            self.toolbar.title.entry.focus()
         elif string == "fullscreen":
             if self.is_fullscreen:
                 self.unfullscreen()
@@ -428,23 +395,29 @@ class Window(Gtk.ApplicationWindow):
         elif string == "quit":
             App().quit(True)
         elif string == "new_page":
-            self.container.add_webview(App().start_page,
-                                       LoadingType.FOREGROUND)
+            self.container.add_webview_for_uri(App().start_page,
+                                               LoadingType.FOREGROUND)
+            self.toolbar.title.start_search()
         elif string == "close_page":
             if self.is_fullscreen:
-                self.container.current.webview.emit("leave-fullscreen")
+                emit_signal(self.container.webview, "leave-fullscreen")
                 Gtk.ApplicationWindow.unfullscreen(self)
-            self.__container.try_close_view(self.container.current)
+            self.__container.try_close_webview(self.container.webview)
         elif string == "reload":
-            self.container.current.webview.reload()
+            self.container.webview.reload()
+        elif string == "settings":
+            # Rework all this code to use actions like in Lollypop
+            from eolie.settings import SettingsDialog
+            dialog = SettingsDialog(self)
+            dialog.show()
         elif string == "home":
-            self.container.current.webview.load_uri(App().start_page)
+            self.container.webview.load_uri(App().start_page)
         elif string == "source":
-            uri = self.container.current.webview.uri
+            uri = self.container.webview.uri
             task_helper = TaskHelper()
             task_helper.load_uri_content(uri, None, self.__on_source_loaded)
         elif string == "find":
-            find_widget = self.container.current.find_widget
+            find_widget = self.container.find_widget
             find_widget.set_search_mode(True)
             find_widget.search()
         elif string == "backward":
@@ -460,7 +433,7 @@ class Window(Gtk.ApplicationWindow):
         elif string == "next_site":
             self.__container.sites_manager.next()
         elif string == "print":
-            self.container.current.webview.print()
+            self.container.webview.print()
         elif string == "private":
             self.container.add_webview(App().start_page,
                                        LoadingType.FOREGROUND,
@@ -468,52 +441,33 @@ class Window(Gtk.ApplicationWindow):
         elif string == "last_page":
             App().pages_menu.activate_last_action()
         elif string == "zoom_in":
-            self.container.current.webview.zoom_in()
+            self.container.webview.zoom_in()
         elif string == "zoom_out":
-            self.container.current.webview.zoom_out()
+            self.container.webview.zoom_out()
         elif string == "zoom_default":
-            self.container.current.webview.zoom_default()
+            self.container.webview.zoom_default()
         elif string == "history":
-            self.toolbar.title.focus_entry("history")
+            self.toolbar.title.entry.focus("history")
         elif string == "search":
-            self.toolbar.title.focus_entry("search")
+            self.toolbar.title.entry.focus("search")
         elif string == "save":
             self.toolbar.end.save_page()
         elif string == "expose":
             active = self.toolbar.actions.view_button.get_active()
             self.toolbar.actions.view_button.set_active(not active)
-        elif string == "jsblock":
-            current_webview = self.container.current.webview
-            App().helper.call("EnableJS",
-                              current_webview.get_page_id(),
-                              GLib.Variant("(s)", (current_webview.netloc,)),
-                              self.__on_js_enabled,
-                              current_webview)
         elif string == "show_sidebar":
             value = App().settings.get_value("show-sidebar")
             App().settings.set_value("show-sidebar",
                                      GLib.Variant("b", not value))
         elif string == "mse_enabled":
-            self.container.current.webview.set_setting("enable-mediasource",
-                                                       True)
-            self.container.current.webview.reload()
-
-    def __on_js_enabled(self, proxy, task, webview):
-        """
-            Reload page
-            @param proxy as Gio.DBusProxy
-            @param task as Gio.Task
-            @param webview as WebView
-        """
-        webview.reload()
+            self.container.webview.set_setting("enable-mediasource", True)
+            self.container.webview.reload()
 
     def __on_popover_closed(self, popover):
         """
             Remove popover from registered
             @param popover as Gtk.Popover
         """
-        # Needed as popover may belong to another class
-        popover.disconnect_by_func(self.__on_popover_closed)
         if popover in self.__popovers:
             self.__popovers.remove(popover)
             GLib.timeout_add(1000, popover.destroy)
